import logging
import time
from typing import List

from botworks.chat.Response import IResponse
from botworks.chat.Trigger import Trigger
from botworks.config_constants import LOG_LEVEL
from botworks.slack.Payload import Payload

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)


class InteractionDefinition:
    def __init__(self, trigger: Trigger, response: IResponse, cooldown_duration=600, mod_exempt=True):
        self.response = response
        self.trigger = trigger
        self.cooldown_duration = cooldown_duration
        self.cooldown_end = 0
        self.modExempt = mod_exempt
        self.conf = {}

    def finalize(self, config):
        self.conf = config
        log.setLevel(config[LOG_LEVEL])

    def check(self, payload: Payload, mod_ids: List[str]) -> bool:
        log.info("Checking for response " + str(self.response))
        if self.modExempt and payload.user in mod_ids:
            return False
        return self.trigger.check(payload)

    def respond(self, clacker, payload: Payload):
        if time.time() < self.cooldown_end:
            return
        self.cooldown_end = time.time() + self.cooldown_duration
        self.response.respond(clacker, payload)


import logging
from difflib import SequenceMatcher

from botworks.slack.Payload import Payload

log = logging.getLogger(__name__)
log.setLevel(logging.WARNING)


class Trigger:
    def __init__(self, match_method=None, match_phrases=None, event=None, full_text=None, full_text_ratio=0.7):
        self.matchMethod = match_method
        self.matchPhrases = match_phrases
        self.event = event
        self.fullText = full_text
        self.fullTextRatio = full_text_ratio

    def check(self, payload: Payload):
        log.debug("Checking " + str(payload.lower_message))
        if self.matchMethod:
            return self.matchMethod.check(payload)
        if self.event == payload.event_type:
            return True
        if self.fullText and payload.imageText and \
                SequenceMatcher(None, self.fullText, payload.imageText).ratio() > self.fullTextRatio:
            return True
        return self.matchPhrases and payload.lower_message and self.matchPhrases.check(payload.lower_message)


class And:
    def __init__(self, *args):
        self.triggers = list(map(lambda x: x.lower() if type(x) == str else x, [*args]))

    def check(self, message) -> bool:
        log.debug("Checking And " + str(len(self.triggers)))
        res = True
        for t in self.triggers:
            if type(t) is str:
                res &= t in message
            else:
                res &= t.check(message)
                if not res:
                    break
        return res


class Or:
    def __init__(self, *args):
        self.triggers = list(map(lambda x: x.lower() if type(x) == str else x, [*args]))

    def check(self, message) -> bool:
        log.debug("Checking Or " + str(len(self.triggers)))
        res = False
        for t in self.triggers:
            if type(t) is str:
                res |= t in message
            else:
                res |= t.check(message)
            if res:
                break
        return res


class Not:
    def __init__(self, *args):
        if len([*args]) != 1:
            log.error("Misconfigured Not, it's going to get weird")
        self.triggers = list(map(lambda x: x.lower() if type(x) == str else x, [*args]))

    def check(self, message) -> bool:
        log.debug("Checking Not " + str(len(self.triggers)))
        if type(self.triggers[0]) is str:
            res = self.triggers[0] not in message
        else:
            res = not self.triggers[0].check(message)

        return res


class StartsWith:
    def __init__(self, *args):
        if len([*args]) != 1:
            log.error("Misconfigured Startswith, it's going to get weird")
        self.triggers = list(map(lambda x: x.lower() if type(x) == str else x, [*args]))

    def check(self, message: str) -> bool:
        log.debug("Checking StartsWith " + str(len(self.triggers)))
        if type(self.triggers[0]) is str:
            return message.startswith(self.triggers[0])

        return False



import unittest
from unittest.mock import create_autospec, call

from botworks.chat.InteractionDefinition import InteractionDefinition
from botworks.chat.Response import Response, Responses
from botworks.chat.Trigger import Or, Trigger
from botworks.slack.Clack import Clack
from botworks.slack.Payload import Payload


def basic_pass(**kwargs):
    pass


def generate_payload(content="hello world"):
    p = Payload()
    p.lower_message = content
    p.channel = "rawr"
    return p


def mock_clack():
    clack_autospec = create_autospec(Clack.__class__)
    clack_autospec.post_message = create_autospec(basic_pass)
    clack_autospec.react = create_autospec(basic_pass)
    clack_autospec.ephemeral_message = create_autospec(basic_pass)
    clack_autospec.thread_reply = create_autospec(basic_pass)
    return clack_autospec


class ResponseTest(unittest.TestCase):

    def test_simpleTrigger_writesSingleTextResponse(self):
        clack_autospec = mock_clack()
        i = InteractionDefinition(
                trigger=Trigger(match_phrases=Or("sample")),
                response=Response(text="sample response")
        )

        i.respond(clack_autospec, generate_payload())
        clack_autospec.post_message.assert_called_once_with(channel='rawr', message='sample response')

    def test_simpleTrigger_writesSingleEmoji(self):
        clack_autospec = mock_clack()
        payload = generate_payload()
        i = InteractionDefinition(
                trigger=Trigger(match_phrases=Or("sample")),
                response=Response(emoji="here")
        )

        i.respond(clack_autospec, payload)
        clack_autospec.react.assert_called_once_with(payload=payload, reaction='here')

    def test_simpleTrigger_writesSingleEphemeral(self):
        clack_autospec = mock_clack()
        payload = generate_payload()
        i = InteractionDefinition(
                trigger=Trigger(match_phrases=Or("sample")),
                response=Response(ephemeral="I'm a ghoooost")
        )

        i.respond(clack_autospec, payload)
        clack_autospec.ephemeral_message.assert_called_once_with(payload=payload, message='I\'m a ghoooost')

    def test_simpleTrigger_writesSingleThreaded(self):
        clack_autospec = mock_clack()
        payload = generate_payload()
        i = InteractionDefinition(
                trigger=Trigger(match_phrases=Or("sample")),
                response=Response(threaded="I'm a threaaaad")
        )

        i.respond(clack_autospec, payload)
        clack_autospec.thread_reply.assert_called_once_with(payload=payload, message="I'm a threaaaad")

    def test_simpleTrigger_writesMultipleEverything(self):
        clack_autospec = mock_clack()
        payload = generate_payload()
        i = InteractionDefinition(
                trigger=Trigger(match_phrases=Or("sample")),
                response=Response(text=["sample response", "response 2"], emoji=["here", "pinged"],
                                  ephemeral=["I'm a ghoooost", "punk"], threaded=["I'm", "a threaaaad"])
        )

        i.respond(clack_autospec, payload)

        self.assertEqual(2, len(clack_autospec.post_message.call_args_list))
        self.assertEqual(clack_autospec.post_message.call_args_list[0], call(message="sample response", channel='rawr'))
        self.assertEqual(clack_autospec.post_message.call_args_list[1], call(message="response 2", channel='rawr'))

        self.assertEqual(2, len(clack_autospec.react.call_args_list))
        self.assertEqual(clack_autospec.react.call_args_list[0], call(reaction="here", payload=payload))
        self.assertEqual(clack_autospec.react.call_args_list[1], call(reaction="pinged", payload=payload))

        self.assertEqual(2, len(clack_autospec.ephemeral_message.call_args_list))
        self.assertEqual(clack_autospec.ephemeral_message.call_args_list[0], call(message="I'm a ghoooost", payload=payload))
        self.assertEqual(clack_autospec.ephemeral_message.call_args_list[1], call(message="punk", payload=payload))

        self.assertEqual(2, len(clack_autospec.thread_reply.call_args_list))
        self.assertEqual(clack_autospec.thread_reply.call_args_list[0], call(message="I'm", payload=payload))
        self.assertEqual(clack_autospec.thread_reply.call_args_list[1], call(message="a threaaaad", payload=payload))

    def test_simpleTrigger_picksFromMultipleResponses(self):
        clack_autospec = mock_clack()
        payload = generate_payload()
        i = InteractionDefinition(
            trigger=Trigger(match_phrases=Or("sample")),
            response=Responses([
                Response(text="sample response"), Response(text="response 2"), Response(text="here"),
                Response(text="pinged"), Response(text="I'm a ghoooost"), Response(text="punk"),
                Response(text="I'm"), Response(text="a threaaaad")])
        )

        i.respond(clack_autospec, payload)
        clack_autospec.post_message.assert_called_once()


if __name__ == '__main__':
    unittest.main()



import logging
import unittest

from botworks.chat.Trigger import Trigger, Or, And
from botworks.slack.Payload import Payload

log = logging.getLogger(__name__)
log.setLevel(logging.WARNING)


def generate_payload(content="hello world"):
    p = Payload()
    p.lower_message = content
    p.channel = "rawr"
    return p


class TriggerTest(unittest.TestCase):

    def test_Phrase_SingleOr(self):
        t = Trigger(match_phrases=Or("Hello"))
        self.assertTrue(t.check(generate_payload()))

    def test_Phrase_SingleAnd(self):
        t = Trigger(match_phrases=And("Hello"))
        self.assertTrue(t.check(generate_payload()))

    def test_Phrase_MultipleAnd(self):
        t = Trigger(match_phrases=And("Hello", "world"))
        self.assertTrue(t.check(generate_payload()))

    def test_Phrase_MultipleOr(self):
        t = Trigger(match_phrases=Or("Hello", "Hola"))
        self.assertTrue(t.check(generate_payload()))

    def test_Phrase_MixedAndOr(self):
        t = Trigger(match_phrases=And(Or("Hello", "Hola"), Or("world", "mundo")))
        self.assertTrue(t.check(generate_payload()))

    def test_Phrase_MixedOrAnd(self):
        t = Trigger(match_phrases=Or(And("Hello", "world"), And("hola", "mundo")))
        self.assertTrue(t.check(generate_payload()))


if __name__ == '__main__':
    unittest.main()



import _thread
import configparser
import logging
import os

from botworks.botworks import Botworks
from botworks.channel.Channel import Channel
from botworks.config_constants import ERROR_SLEEP_TIME, SLEEP_TIME, LOG_LEVEL
from sampleconfig import gameofthrones

config = configparser.ConfigParser().read("config.ini")

logging.basicConfig()
log = logging.getLogger(__name__)
log.setLevel(logging.WARNING)


def listen_thread():
    got_channel = Channel(name="gameofthrones", mod_names=["Syrio Forel", "threeeyedraven"],
                          responses=gameofthrones.messages)
    Botworks(
        str(os.environ.get('SLACK_BOT_TOKEN')),
        str(os.environ.get('SLACK_BOT_NAME')),
        [got_channel],
        config={
            LOG_LEVEL: logging.WARNING,
            SLEEP_TIME: 3,
            ERROR_SLEEP_TIME: 8
        }
    ).listen()


if __name__ == "__main__":
    log.error("Starting")
    _thread.start_new_thread(listen_thread, ())
    log.error("Listener thread started")



import logging
import unittest
from unittest.mock import MagicMock

from botworks.chat.Response import Response
from botworks.chat.Trigger import Trigger, Or, And
from botworks.slack.Clack import Clack
from botworks.slack.Payload import Payload

log = logging.getLogger(__name__)
log.setLevel(logging.WARNING)


def mock_clack():
    return MagicMock(Clack)


def generate_payload(content="hello world"):
    p = Payload()
    p.lower_message = content
    p.channel = "rawr"
    return p


class ResponseTest(unittest.TestCase):

    def test_textResponse_singleResponse(self):
        c = mock_clack()
        r = Response(text="Hi back")
        r.respond(c, generate_payload())

        c.assert_called_once()

if __name__ == '__main__':
    unittest.main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("version.txt", "r") as vf:
    version = vf.readline()

setuptools.setup(
    name="botworks",
    version=version,
    author="William Hanson",
    author_email="42045551+doubleyuhtee@users.noreply.github.com",
    description="Slack bot framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/doubleyuhtee/botworks",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta"
    ),
)